﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Adams.Admin.Business.Console
{
    public static class CreateUser
    {
        public static Adams.Admin.DataTransfer.Console.CreateUser.PageLoad PageLoad(int id)
        {
            var model = DataAccess.Console.CreateUser.PageLoad(id);
            if(model.User.ID == 0)
            {
                model.User.IsActive = true;
            }
            return model;
        }

        public static DataTransfer.FormOutput SaveUser(DataTransfer.Console.CreateUser.FormInput input)
        {
            var output = Validation.PerformBasicValidation(input);

            if (output.IsValid)
            {
                var userinfo = DataAccess.Console.CreateUser.GetUserInfo(input.ID.ToInt(), input.Email);
                if (input.ID.ToInt() > 0)
                {
                    if (!Shared.Str.IsEqual(input.Email, userinfo.Email))
                    {
                        if (userinfo.Exists)
                        {
                            output.InvalidFields.Add(new DataTransfer.InvalidField()
                            {
                                Name = "Email",
                                Code = 4
                            });
                        }
                    }
                }
                else
                {
                    if (userinfo.Exists)
                    {
                        output.InvalidFields.Add(new DataTransfer.InvalidField()
                        {
                            Name = "Email",
                            Code = 4
                        });
                    }

                    if (string.IsNullOrEmpty(input.Password))
                    {
                        output.InvalidFields.Add(new DataTransfer.InvalidField()
                        {
                            Name = "Password",
                            Code = 1
                        });
                    }
                }

                if (output.IsValid)
                {
                    string passwordHash = "";
                    string passwordSalt = "";

                    if (input.ID.ToInt() == 0)
                    {
                        passwordSalt = Shared.Encryption.GenerateSalt(16);
                        passwordHash = Shared.Encryption.HashPassword(input.Password, passwordSalt);
                    }
                    else
                    {
                        if (string.IsNullOrEmpty(input.Password))
                        {
                            passwordHash = userinfo.PasswordHash;
                            passwordSalt = userinfo.PasswordSalt;
                        }
                        else
                        {
                            passwordHash = Adams.Shared.Encryption.HashPassword(input.Password, userinfo.PasswordSalt);
                            passwordSalt = userinfo.PasswordSalt;
                        }
                    }

                    var saveinfo = new DataTransfer.Console.CreateUser.SaveUser()
                    {
                        User = new Shared.Entities.Console.User()
                        {
                            ID = input.ID.ToInt(),
                            FirstName = input.FirstName,
                            LastName = input.LastName,
                            Email = input.Email,
                            IsActive = input.IsActive,
                            IsLocked = input.IsLocked
                        },
                        PasswordHash = passwordHash,
                        PasswordSalt = passwordSalt
                    };

                    saveinfo.User.Applications = new Shared.Entities.Console.ApplicationCollection();
                    if (input.Applications != null)
                    {
                        foreach (var app in input.Applications)
                        {
                            saveinfo.User.Applications.Add(new Shared.Entities.Console.Application()
                            {
                                ID = app
                            });
                        }
                    }

                    DataAccess.Console.CreateUser.SaveUser(saveinfo);
                    output.IsSaved = true;
                }
            }

            return output;
        }
    }
}
