﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Data;
namespace Adams.Admin.DataAccess.DailyPrayers
{
    public static class CreateLocation
    {
        public static Adams.Admin.DataTransfer.DailyPrayers.CreateLocation.PageLoad PageLoad(int id)
        {
            var data = new Adams.Admin.DataTransfer.DailyPrayers.CreateLocation.PageLoad();
            data.Location = new Shared.Entities.DailyPrayers.Location();

            using (SqlDataReader dr = DB.RunDataReader(@"
IF(@LocationID > 0)
BEGIN
    SELECT * FROM DailyPrayers.Locations WHERE LocationID = @LocationID
END

SELECT COUNT(*) AS MasterCount FROM DailyPrayers.Locations WHERE IsMaster = 1
", CommandType.Text,
            DB.CreateParameter("@LocationID", id, SqlDbType.Int)))
            {
                if (id > 0)
                {
                    dr.Read();
                    data.Location.ID = dr["LocationID"].DbInt();
                    data.Location.IsMaster = dr["IsMaster"].DbBool();
                    data.Location.InheritMaster = dr["InheritMaster"].DbBool();
                    data.Location.Name = dr["Name"].DbString();
                    data.Location.Street = dr["Street"].DbString();
                    data.Location.Street2 = dr["Street2"].DbString();
                    data.Location.City = dr["City"].DbString();
                    data.Location.State = dr["State"].DbString();
                    data.Location.ZipCode = dr["ZipCode"].DbString();
                    data.Location.Latitude = dr["Latitude"].DbDecimal();
                    data.Location.Longitude = dr["Longitude"].DbDecimal();
                    data.Location.TimeZone = dr["TimeZone"].DbInt();
                    data.Location.LongNote = dr["LongNote"].DbString();
                    data.Location.DisplayOrder = dr["DisplayOrder"].DbInt();
                    dr.NextResult();
                }

                dr.Read();
                data.IsMasterAssigned = dr["MasterCount"].DbInt() > 0;
            }

            return data;
        }

        public static void Save(DataTransfer.DailyPrayers.CreateLocation.FormInput input)
        {
            DB.Run(@"
IF(@LocationID > 0)
BEGIN
    UPDATE DailyPrayers.Locations SET
        IsMaster = @IsMaster,
        InheritMaster = @InheritMaster,
        Name = @Name,
        Street = @Street,
        Street2 = @Street2,
        City = @City,
        State = @State,
        ZipCode = @ZipCode,
        Latitude = @Latitude,
        Longitude = @Longitude,
        TimeZone = @TimeZone,
        LongNote = @LongNote,
        DisplayOrder = @DisplayOrder
    WHERE LocationID = @LocationID
END
ELSE
BEGIN
    INSERT INTO DailyPrayers.Locations (
        IsMaster,
        InheritMaster,
        Name,
        Street,
        Street2,
        City,
        State,
        ZipCode,
        Latitude,
        Longitude,
        TimeZone,
        LongNote,
        DisplayOrder)
    VALUES (
        @IsMaster,
        @InheritMaster,
        @Name,
        @Street,
        @Street2,
        @City,
        @State,
        @ZipCode,
        @Latitude,
        @Longitude,
        @TimeZone,
        @LongNote,
        @DisplayOrder)
END
    ", CommandType.Text,
                    DB.CreateParameter("@LocationID", input.ID.ToInt(), SqlDbType.Int),
                    DB.CreateParameter("@Name", input.Name.ToNString(), SqlDbType.VarChar, 50),
                    DB.CreateParameter("@IsMaster", input.IsMaster, SqlDbType.Bit),
                    DB.CreateParameter("@InheritMaster", input.InheritMaster, SqlDbType.Bit),
                    DB.CreateParameter("@Street", input.Street.ToNString(), SqlDbType.VarChar, 100),
                    DB.CreateParameter("@Street2", input.Street2.ToNString(), SqlDbType.VarChar, 100),
                    DB.CreateParameter("@City", input.City.ToNString(), SqlDbType.VarChar, 50),
                    DB.CreateParameter("@State", input.State.ToNString(), SqlDbType.VarChar, 2),
                    DB.CreateParameter("@ZipCode", input.ZipCode.ToNString(), SqlDbType.VarChar, 10),
                    DB.CreateParameter("@Latitude", input.Latitude.ToNString(), SqlDbType.Decimal),
                    DB.CreateParameter("@Longitude", input.Longitude.ToNString(), SqlDbType.Decimal),
                    DB.CreateParameter("@TimeZone", input.TimeZone.ToNString(), SqlDbType.Int),
                    DB.CreateParameter("@LongNote", input.LongNote.ToNString(), SqlDbType.Text),
                    DB.CreateParameter("@DisplayOrder", input.DisplayOrder.ToNString(), SqlDbType.Int));
        }

        public static int GetCurrentMaster()
        {
            int locationid = DB.RunScalar(@"SELECT LocationID FROM DailyPrayers.Locations WHERE IsMaster = 1").DbInt();
            return locationid;
        }
    }
}
