﻿using System;
using System.IO;
using System.Security.Cryptography;

namespace Adams.Public.Api
{
    public class Email
    {
        public static void QueueEmail(string subject, string from, string to, string body, int priority)
        {
            DataAccess.Email.QueueEmail(subject, from, to, body, priority);      
        }

        public static string GetEmailFromCode(string code)
        {
            string en = Shared.Crypt.Decrypt(code);
            string email = en.Split('|')[0];
            return email;
        }

        public static string GenerateSignupEmail(string email)
        {
            string templatepath = Shared.Config.GetAppSetting("EmailTemplatesPath").TrimEnd('\\') + "\\signup.htm";
            string body = Shared.IOHelper.GetFileContents(templatepath);
            string code = Shared.Crypt.Encrypt(email + "|" + DateTime.Now.ToString());
            code = System.Web.HttpUtility.UrlEncode(code);
            string url = Shared.Config.GetAppSetting("BaseUrl").TrimEnd('/') + "/Security/ConfirmEmail?Code=" + code;
            body = body.Replace("[URL]", url);
            body = body.Replace("[EMAIL]", email);
            return body;
        }

        public static string GenerateForgotEmail(string email)
        {
            string templatepath = Shared.Config.GetAppSetting("EmailTemplatesPath").TrimEnd('\\') + "\\forgot.htm";
            string body = Shared.IOHelper.GetFileContents(templatepath);
            string code = Shared.Crypt.Encrypt(email + "|" + DateTime.Now.ToString());
            code = System.Web.HttpUtility.UrlEncode(code);
            string url = Shared.Config.GetAppSetting("BaseUrl").TrimEnd('/') + "/Security/PasswordReset?Code=" + code;
            body = body.Replace("[URL]", url);
            body = body.Replace("[EMAIL]", email);
            return body;
        }

        public static string GeneratePasswordResetCompleteEmail(string email)
        {
            string templatepath = Shared.Config.GetAppSetting("EmailTemplatesPath").TrimEnd('\\') + "\\forgotcomplete.htm";
            string body = Shared.IOHelper.GetFileContents(templatepath);
            body = body.Replace("[EMAIL]", email);
            return body;
        }

        public static string GenerateChangeEmail(string email)
        {
            string templatepath = Shared.Config.GetAppSetting("EmailTemplatesPath").TrimEnd('\\') + "\\changeemail.htm";
            string body = Shared.IOHelper.GetFileContents(templatepath);
            string code = Shared.Crypt.Encrypt(email + "|" + DateTime.Now.ToString());
            code = System.Web.HttpUtility.UrlEncode(code);
            string url = Shared.Config.GetAppSetting("BaseUrl").TrimEnd('/') + "/Security/ConfirmEmail?Code=" + code;
            body = body.Replace("[URL]", url);
            body = body.Replace("[EMAIL]", email);
            return body;
        }

        public static void SendRecurringDonationEmail(int donationId)
        {
            var donation = DataAccess.Donations.GetRecurring(donationId);

            string subject = "Thank you for your recurring donation!";
            string from = Shared.Config.GetAppSetting("SecurityFromEmail");
            string templatepath = Shared.Config.GetAppSetting("EmailTemplatesPath").TrimEnd('\\') + "\\recurringdonate.htm";    

            string body = Shared.IOHelper.GetFileContents(templatepath);
            body = body.Replace("[BUCKETNAME]", donation.Bucket.Name);
            body = body.Replace("[AMOUNT]", string.Format("{0:C}", donation.Amount));
            body = body.Replace("[FIRSTNAME]", donation.Member.FirstName);
            body = body.Replace("[LASTNAME]", donation.Member.LastName);
            
            Email.QueueEmail(subject, from, donation.Member.Email, body, 1);
        }

        public static void SendOneDonationEmail(int donationId)
        {
            var donation = DataAccess.Donations.GetOneTime(donationId);

            string subject = "Thank you for your donation!";
            string from = Shared.Config.GetAppSetting("SecurityFromEmail");
            string templatepath = Shared.Config.GetAppSetting("EmailTemplatesPath").TrimEnd('\\') + "\\onetimedonate.htm";

            string body = Shared.IOHelper.GetFileContents(templatepath);
            body = body.Replace("[BUCKETNAME]", donation.Bucket.Name);
            body = body.Replace("[AMOUNT]", string.Format("{0:C}", donation.Amount));
            body = body.Replace("[FIRSTNAME]", donation.Member.FirstName);
            body = body.Replace("[LASTNAME]", donation.Member.LastName);

            Email.QueueEmail(subject, from, donation.Member.Email, body, 1);
        }

        public static void SendQuickDonationEmail(int donationId)
        {
            var donation = DataAccess.Donations.GetQuick(donationId);

            string subject = "Thank you for your donation!";
            string from = Shared.Config.GetAppSetting("SecurityFromEmail");
            string templatepath = Shared.Config.GetAppSetting("EmailTemplatesPath").TrimEnd('\\') + "\\quickdonate.htm";

            string body = Shared.IOHelper.GetFileContents(templatepath);
            body = body.Replace("[BUCKETNAME]", donation.Bucket.Name);
            body = body.Replace("[AMOUNT]", string.Format("{0:C}", donation.Amount));
            body = body.Replace("[FIRSTNAME]", donation.FirstName);
            body = body.Replace("[LASTNAME]", donation.LastName);

            Email.QueueEmail(subject, from, donation.Email, body, 1);
        }
    }
}