﻿using System.Data;
using System.Data.SqlClient;
using System.Linq;

namespace Adams.Public.DataAccess
{
    public static class Classifieds
    {
        public static Adams.Public.DataTransfer.Classifieds.TypeCollection GetTypesAndCategories()
        {
            var data = new Adams.Public.DataTransfer.Classifieds.TypeCollection();

            using (SqlDataReader dr = DB.RunDataReader(@"
SELECT * FROM Classifieds.Types ORDER BY Name
SELECT C.*, (SELECT COUNT(*) FROM Classifieds.Postings P WHERE P.CategoryID = C.CategoryID AND P.StatusID = 2 AND DATEDIFF(d, GETDATE(), P.ExpiresOn) >= 0) AS Postings FROM Classifieds.Categories C ORDER BY C.Name
", CommandType.Text))
            {
                while (dr.Read())
                {
                    var type = new Adams.Public.DataTransfer.Classifieds.Type();
                    type.Categories2 = new Adams.Public.DataTransfer.Classifieds.CategoryCollection();
                    type.ID = dr["TypeID"].DbInt();
                    type.Name = dr["Name"].DbString();
                    data.Add(type);
                }
                dr.NextResult();
                while (dr.Read())
                {
                    var cat = new Adams.Public.DataTransfer.Classifieds.Category();
                    cat.ID = dr["CategoryID"].DbInt();
                    cat.Name = dr["Name"].DbString();
                    cat.Postings = dr["Postings"].DbInt();
                    data.Where(t => t.ID == dr["TypeID"].DbInt()).FirstOrDefault().Categories2.Add(cat);
                }
            }

            return data;
        }

        public static Adams.Shared.Entities.Classifieds.PostingCollection ListPostings(Adams.Public.DataTransfer.Classifieds.ListPostingsInput input)
        {
            var data = new Adams.Shared.Entities.Classifieds.PostingCollection();

            using (SqlDataReader dr = DB.RunDataReader(@"
SELECT P.PostingID,P.CategoryID,P.StatusID,P.Title,P.Price,P.Location,P.ReplyTo,P.Description,P.CreatedOn,P.ExpiresOn,C.Name As Category,T.TypeID AS TypeID, T.Name AS Type
FROM Classifieds.Postings P 
INNER JOIN Classifieds.Categories C ON P.CategoryID = C.CategoryID 
INNER JOIN Classifieds.Types T ON C.TypeID = T.TypeID 
WHERE T.Name = @Type" + (string.IsNullOrEmpty(input.Category) ? "" : @" AND C.Name = @Category") + @"
AND P.StatusID = 2 AND DATEDIFF(d, GETDATE(), P.ExpiresOn) >= 0
ORDER BY ExpiresOn DESC
", CommandType.Text,
    DB.CreateParameter("@Type", input.Type.ToNString()),
    DB.CreateParameter("@Category", input.Category.ToNString())))
            {
                while (dr.Read())
                {
                    var posting = new Shared.Entities.Classifieds.Posting();
                    posting.ID = dr["PostingID"].DbInt();
                    posting.Status = (Shared.Entities.Classifieds.Status)dr["StatusID"].DbInt();
                    posting.Title = dr["Title"].DbString();
                    posting.Price = dr["Price"].DbString();
                    posting.Location = dr["Location"].DbString();
                    posting.ReplyTo = dr["ReplyTo"].DbString();
                    posting.Description = dr["Description"].DbString();
                    posting.CreatedOn = dr["CreatedOn"].DbDate();
                    posting.ExpiresOn = dr["ExpiresOn"].DbDate();
                    data.Add(posting);
                }
            }

            return data;
        }

        public static Adams.Shared.Entities.Classifieds.PostingCollection SearchPostings(Adams.Public.DataTransfer.Classifieds.SearchPostingsInput input)
        {
            var data = new Adams.Shared.Entities.Classifieds.PostingCollection();

            using (SqlDataReader dr = DB.RunDataReader(@"
SELECT P.PostingID,P.CategoryID,P.StatusID,P.Title,P.Price,P.Location,P.ReplyTo,P.Description,P.CreatedOn,P.ExpiresOn,C.Name As Category,T.TypeID AS TypeID, T.Name AS Type
FROM Classifieds.Postings P 
INNER JOIN Classifieds.Categories C ON P.CategoryID = C.CategoryID 
INNER JOIN Classifieds.Types T ON C.TypeID = T.TypeID 
WHERE FREETEXT ((Title,Price,Location,Description), '" + input.Search.Replace("'", "''") + @"' )
AND P.StatusID =2  AND DATEDIFF(d, GETDATE(), P.ExpiresOn) >= 0
ORDER BY ExpiresOn DESC
", CommandType.Text,
    DB.CreateParameter("@Search", input.Search.ToNString())))
            {
                while (dr.Read())
                {
                    var posting = new Shared.Entities.Classifieds.Posting();
                    posting.ID = dr["PostingID"].DbInt();
                    posting.Status = (Shared.Entities.Classifieds.Status)dr["StatusID"].DbInt();
                    posting.Title = dr["Title"].DbString();
                    posting.Price = dr["Price"].DbString();
                    posting.Location = dr["Location"].DbString();
                    posting.ReplyTo = dr["ReplyTo"].DbString();
                    posting.Description = dr["Description"].DbString();
                    posting.CreatedOn = dr["CreatedOn"].DbDate();
                    posting.ExpiresOn = dr["ExpiresOn"].DbDate();

                    posting.Category = new Shared.Entities.Classifieds.Category()
                    {
                        ID = dr["CategoryID"].DbInt(),
                        Name = dr["Type"].DbString(),
                        Type = new Shared.Entities.Classifieds.Type()
                        {
                            ID = dr["TypeID"].DbInt(),
                            Name = dr["Type"].DbString()
                        }
                    };

                    data.Add(posting);
                }
            }

            return data;
        }

        public static Adams.Shared.Entities.Classifieds.Posting Posting(Adams.Public.DataTransfer.Classifieds.PostingInput input)
        {
            var data = new Adams.Shared.Entities.Classifieds.Posting();

            using (SqlDataReader dr = DB.RunDataReader(@"
SELECT P.PostingID,P.CategoryID,P.StatusID,P.Title,P.Price,P.Location,P.ReplyTo,P.Description,P.CreatedOn,P.ExpiresOn,C.Name As Category,T.TypeID AS TypeID, T.Name AS Type
FROM Classifieds.Postings P 
INNER JOIN Classifieds.Categories C ON P.CategoryID = C.CategoryID 
INNER JOIN Classifieds.Types T ON C.TypeID = T.TypeID 
WHERE PostingID = @PostingID
", CommandType.Text,
    DB.CreateParameter("@PostingID", input.PostingID)))
            {
                dr.Read();
                data.ID = dr["PostingID"].DbInt();
                data.Status = (Shared.Entities.Classifieds.Status)dr["StatusID"].DbInt();
                data.Title = dr["Title"].DbString();
                data.Price = dr["Price"].DbString();
                data.Location = dr["Location"].DbString();
                data.ReplyTo = dr["ReplyTo"].DbString();
                data.Description = dr["Description"].DbString();
                data.CreatedOn = dr["CreatedOn"].DbDate();
                data.ExpiresOn = dr["ExpiresOn"].DbDate();

                data.Category = new Shared.Entities.Classifieds.Category()
                {
                    ID = dr["CategoryID"].DbInt(),
                    Name = dr["Type"].DbString(),
                    Type = new Shared.Entities.Classifieds.Type()
                    {
                        ID = dr["TypeID"].DbInt(),
                        Name = dr["Type"].DbString()
                    }
                };
            }

            return data;
        }

        public static Adams.Public.DataTransfer.Classifieds.GetMemberPostings GetMemberPostings(Adams.Public.DataTransfer.Classifieds.GetMemberPostingsInput input)
        {
            var data = new Adams.Public.DataTransfer.Classifieds.GetMemberPostings();
            data.Postings = new Shared.Entities.Classifieds.PostingCollection();

            using (SqlDataReader dr = DB.RunDataReader(@"
SELECT P.PostingID,P.CategoryID,P.MemberID,P.StatusID,P.Title,P.Price,P.Location,P.ReplyTo,P.Description,P.CreatedOn,P.ExpiresOn,M.FirstName,M.LastName,M.Email,C.Name As Category,T.TypeID AS TypeID, T.Name AS Type
FROM Classifieds.Postings P 
INNER JOIN Classifieds.Categories C ON P.CategoryID = C.CategoryID 
INNER JOIN Classifieds.Types T ON C.TypeID = T.TypeID 
INNER JOIN Membership.Members M ON P.MemberID = M.MemberID
WHERE P.MemberID = @MemberID 
AND P.StatusID <> 4 AND DATEDIFF(d, GETDATE(), P.ExpiresOn) >= -7
ORDER BY ExpiresOn ASC
", CommandType.Text,
    DB.CreateParameter("@MemberID", input.MemberID)))
            {
                while (dr.Read())
                {
                    var posting = new Shared.Entities.Classifieds.Posting();
                    posting.ID = dr["PostingID"].DbInt();
                    posting.Status = (Shared.Entities.Classifieds.Status)dr["StatusID"].DbInt();
                    posting.Title = dr["Title"].DbString();
                    posting.Price = dr["Price"].DbString();
                    posting.Location = dr["Location"].DbString();
                    posting.ReplyTo = dr["ReplyTo"].DbString();
                    posting.Description = dr["Description"].DbString();
                    posting.CreatedOn = dr["CreatedOn"].DbDate();
                    posting.ExpiresOn = dr["ExpiresOn"].DbDate();

                    posting.Category = new Shared.Entities.Classifieds.Category()
                    {
                        ID = dr["CategoryID"].DbInt(),
                        Name = dr["Type"].DbString(),
                        Type = new Shared.Entities.Classifieds.Type()
                        {
                            ID = dr["TypeID"].DbInt(),
                            Name = dr["Type"].DbString()
                        }
                    };

                    posting.Member = new Shared.Entities.Membership.Member()
                    {
                        ID = dr["MemberID"].DbInt(),
                        Email = dr["Email"].DbString(),
                        FirstName = dr["FirstName"].DbString(),
                        LastName = dr["LastName"].DbString(),
                    };

                    data.Postings.Add(posting);
                }
            }

            return data;            
        }

        public static void Renew(Adams.Public.DataTransfer.Classifieds.RenewSave input)
        {
            DB.Run(@"
UPDATE Classifieds.Postings
SET ExpiresOn = @ExpiresOn
WHERE PostingID = @PostingID AND MemberID = @MemberID
", CommandType.Text,
                    DB.CreateParameter("@PostingID", input.PostingID, SqlDbType.Int),
                    DB.CreateParameter("@MemberID", input.MemberID, SqlDbType.Int),
                    DB.CreateParameter("@ExpiresOn", input.ExpiresOn));
        }

        public static void Remove(Adams.Public.DataTransfer.Classifieds.RemoveInput input)
        {
            DB.Run(@"
UPDATE Classifieds.Postings
SET StatusID = 4
WHERE PostingID = @PostingID AND MemberID = @MemberID
", CommandType.Text,
                    DB.CreateParameter("@PostingID", input.PostingID, SqlDbType.Int),
                    DB.CreateParameter("@MemberID", input.MemberID, SqlDbType.Int));
        }
    }
}
