﻿using System.Data;
using System.Data.SqlClient;
using System.Linq;

namespace Adams.Public.DataAccess
{
    public static class JummahPrayers
    {
        public static Adams.Shared.Entities.JummahPrayers.LocationCollection GetLocations()
        {
            var data = new Shared.Entities.JummahPrayers.LocationCollection();

            using (SqlDataReader dr = DB.RunDataReader(@"
SELECT LocationID, Name, Street, Street2, City, State, ZipCode, Latitude, Longitude, DisplayOrder
FROM JummahPrayers.Locations
ORDER BY DisplayOrder

SELECT *
FROM JummahPrayers.Prayers
ORDER BY DisplayOrder
", CommandType.Text))
            {

                while (dr.Read())
                {
                    var location = new Shared.Entities.JummahPrayers.Location();
                    location.Prayers = new Shared.Entities.JummahPrayers.PrayerCollection();
                    location.ID = dr["LocationID"].DbInt();
                    location.Name = dr["Name"].DbString();
                    location.Street = dr["Street"].DbString();
                    location.Street2 = dr["Street2"].DbString();
                    location.City = dr["City"].DbString();
                    location.State = dr["State"].DbString();
                    location.ZipCode = dr["ZipCode"].DbString();
                    location.Latitude = dr["Latitude"].DbDecimal();
                    location.Longitude = dr["Longitude"].DbDecimal();
                    location.DisplayOrder = dr["DisplayOrder"].DbInt();
                    data.Add(location);
                }

                dr.NextResult();

                while (dr.Read())
                {
                    var prayer = new Shared.Entities.JummahPrayers.Prayer();
                    prayer.ID = dr["PrayerID"].DbInt();
                    prayer.Name = dr["Name"].DbString();
                    prayer.Time = dr["Time"].DbString();
                    prayer.DisplayOrder = dr["DisplayOrder"].DbInt();
                    data.Where(l => l.ID == dr["LocationID"].DbInt()).FirstOrDefault().Prayers.Add(prayer);
                }

                return data;
            }
        }

        public static Adams.Shared.Entities.JummahPrayers.Location GetLocation(Adams.Public.DataTransfer.JummahPrayers.GetLocationInput input)
        {
            var data = new Shared.Entities.JummahPrayers.Location();
            data.Prayers = new Shared.Entities.JummahPrayers.PrayerCollection();

            using (SqlDataReader dr = DB.RunDataReader(@"
DECLARE @LocationID int
SELECT @LocationID = LocationID FROM JummahPrayers.Locations WHERE Name = @Location

SELECT LocationID, Name, Street, Street2, City, State, ZipCode, Latitude, Longitude, DisplayOrder, FridayNote, LongNote
FROM JummahPrayers.Locations
WHERE LocationID = @LocationID

SELECT *
FROM JummahPrayers.Prayers
WHERE LocationID = @LocationID
ORDER BY DisplayOrder
", CommandType.Text,
    DB.CreateParameter("@Location", input.Location)))
            {

                dr.Read();
                data.ID = dr["LocationID"].DbInt();
                data.Name = dr["Name"].DbString();
                data.Street = dr["Street"].DbString();
                data.Street2 = dr["Street2"].DbString();
                data.City = dr["City"].DbString();
                data.State = dr["State"].DbString();
                data.ZipCode = dr["ZipCode"].DbString();
                data.Latitude = dr["Latitude"].DbDecimal();
                data.Longitude = dr["Longitude"].DbDecimal();
                data.FridayNote = dr["FridayNote"].DbString();
                data.LongNote = dr["LongNote"].DbString();
                data.DisplayOrder = dr["DisplayOrder"].DbInt();

                dr.NextResult();

                while (dr.Read())
                {
                    var prayer = new Shared.Entities.JummahPrayers.Prayer();
                    prayer.ID = dr["PrayerID"].DbInt();
                    prayer.Name = dr["Name"].DbString();
                    prayer.Time = dr["Time"].DbString();
                    prayer.DisplayOrder = dr["DisplayOrder"].DbInt();
                    data.Prayers.Add(prayer);
                }

                return data;
            }
        }
    }
}
