﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Data;

namespace Adams.Public.DataAccess.Web
{
    public static class Layout
    {
        public static Adams.Public.DataTransfer.Web.Layout.LayoutPageLoad LayoutPageLoad(Adams.Public.DataTransfer.Web.Layout.LayoutPageLoadInputBusiness input)
        {
            var data = new Adams.Public.DataTransfer.Web.Layout.LayoutPageLoad();
            var topnodes = new Shared.Entities.Sitemap.WebTopNodeCollection();
            var bottomnodes = new Shared.Entities.Sitemap.WebBottomNodeCollection();
            data.BreadCrumb = new Shared.Entities.Sitemap.NodeCollection();

            using (SqlDataReader dr = DB.RunDataReader(@"
IF(@MemberID > 0)
BEGIN
    SELECT MemberID, FirstName, LastName, Email 
    FROM Membership.Members
    WHERE MemberID = @MemberID
END;

-- Top Nodes
WITH AllNodes AS (
	SELECT n.NodeID, n.ParentNodeID, n.DisplayOrder, 0 AS [Level], n.Title, n.Url
	FROM Sitemap.WebTopNodes n
	WHERE NodeID = 1
	UNION ALL
	SELECT n.NodeID, n.ParentNodeID, n.DisplayOrder, a.[Level] + 1, n.Title, n.Url
	FROM Sitemap.WebTopNodes n JOIN AllNodes a ON n.ParentNodeID = a.NodeID 
)
SELECT * 
FROM AllNodes
ORDER BY Level, DisplayOrder;

-- Bottom Nodes
WITH AllNodes AS (
	SELECT n.NodeID, n.ParentNodeID, n.DisplayOrder, 0 AS [Level], n.Title, n.Url
	FROM Sitemap.WebBottomNodes n
	WHERE NodeID = 1
	UNION ALL
	SELECT n.NodeID, n.ParentNodeID, n.DisplayOrder, a.[Level] + 1, n.Title, n.Url
	FROM Sitemap.WebBottomNodes n JOIN AllNodes a ON n.ParentNodeID = a.NodeID 
)
SELECT * 
FROM AllNodes
ORDER BY Level, DisplayOrder;

-- Left Nodes
DECLARE @NodeID int
DECLARE @ParentNodeID int
DECLARE @ParentParentNodeID int
SELECT @NodeID = NodeID, @ParentNodeID = ParentNodeID FROM Sitemap.WebLeftNodes n WHERE n.Url = @Url

IF(@NodeID IS NULL)
BEGIN
    SELECT TOP 1 @NodeID = NodeID, @ParentNodeID = ParentNodeID FROM Sitemap.WebLeftNodes n WHERE n.Url <> '/' AND n.Url <> '/myadams' AND CHARINDEX(n.Url, @Url) > 0
END

SELECT @ParentParentNodeID = ParentNodeID FROM Sitemap.WebLeftNodes n WHERE n.NodeID = @ParentNodeID

SELECT n.* FROM Sitemap.WebLeftNodes n WHERE n.NodeID = @NodeID
SELECT n.* FROM Sitemap.WebLeftNodes n WHERE n.NodeID = @ParentNodeID
SELECT n.* FROM Sitemap.WebLeftNodes n WHERE n.ParentNodeID = @ParentNodeID ORDER BY DisplayOrder;
SELECT n.* FROM Sitemap.WebLeftNodes n WHERE n.ParentNodeID = @NodeID ORDER BY DisplayOrder;
SELECT n.* FROM Sitemap.WebLeftNodes n WHERE n.ParentNodeID = @ParentParentNodeID ORDER BY DisplayOrder;

-- BreadCrumb Nodes
SELECT @NodeID = NodeID FROM Sitemap.Nodes n WHERE n.Url = @Url

IF(@NodeID IS NULL)
BEGIN
    SELECT TOP 1 @NodeID = NodeID FROM Sitemap.Nodes n WHERE n.Url <> '/' AND n.Url <> '/myadams' AND CHARINDEX(n.Url, @Url) > 0
END;

WITH AllNodes AS (
	SELECT n.NodeID, n.ParentNodeID, 0 AS [Level], n.Title, n.Url
	FROM Sitemap.Nodes n
	WHERE n.NodeID = @NodeID
	UNION ALL
	SELECT n.NodeID, n.ParentNodeID, a.[Level] + 1, n.Title, n.Url
	FROM Sitemap.Nodes n JOIN AllNodes a ON n.NodeID = a.ParentNodeID 
)
SELECT * 
FROM AllNodes
ORDER BY Level DESC;
",
                   CommandType.Text,
                   DB.CreateParameter("@MemberID", input.MemberID),
                   DB.CreateParameter("@Url", input.Url)))
            {
                if (input.MemberID > 0)
                {
                    dr.Read();
                    data.FirstName = dr["FirstName"].DbString();
                    data.LastName = dr["LastName"].DbString();
                    data.Email = dr["Email"].DbString();
                    dr.NextResult();
                }

                // Top Nav
                while (dr.Read())
                {
                    var node = new Shared.Entities.Sitemap.WebTopNode();
                    node.ID = dr["NodeID"].DbInt();
                    node.Url = dr["Url"].DbString();
                    node.Title = dr["Title"].DbString();
                    node.DisplayOrder = dr["DisplayOrder"].DbInt();
                    node.ParentNode = new Shared.Entities.Sitemap.WebTopNode()
                    {
                        ID = dr["ParentNodeID"].DbInt()
                    };
                    node.ChildNodes = new Shared.Entities.Sitemap.WebTopNodeCollection();
                    topnodes.Add(node);
                }

                // Bottom Nav
                dr.NextResult();
                while (dr.Read())
                {
                    var node = new Shared.Entities.Sitemap.WebBottomNode();
                    node.ID = dr["NodeID"].DbInt();
                    node.Url = dr["Url"].DbString();
                    node.Title = dr["Title"].DbString();
                    node.DisplayOrder = dr["DisplayOrder"].DbInt();
                    node.ParentNode = new Shared.Entities.Sitemap.WebBottomNode()
                    {
                        ID = dr["ParentNodeID"].DbInt()
                    };
                    node.ChildNodes = new Shared.Entities.Sitemap.WebBottomNodeCollection();
                    bottomnodes.Add(node);
                }

                // Left Nav
                dr.NextResult();
                data.CurrentLeftNode = new Shared.Entities.Sitemap.WebLeftNode()
                {
                    ChildNodes = new Shared.Entities.Sitemap.WebLeftNodeCollection(),
                    ParentNode = new Shared.Entities.Sitemap.WebLeftNode() {
                        ChildNodes = new Shared.Entities.Sitemap.WebLeftNodeCollection(),
                        ParentNode = new Shared.Entities.Sitemap.WebLeftNode()
                        {
                            ChildNodes = new Shared.Entities.Sitemap.WebLeftNodeCollection()
                        }
                    }
                };

                if (dr.HasRows)
                {
                    dr.Read();
                    data.CurrentLeftNode.ID = dr["NodeID"].DbInt();
                    data.CurrentLeftNode.Url = dr["Url"].DbString();
                    data.CurrentLeftNode.Title = dr["Title"].DbString();
                    data.CurrentLeftNode.DisplayOrder = dr["DisplayOrder"].DbInt();
                }

                dr.NextResult();
                if (dr.HasRows)
                {
                    dr.Read();
                    data.CurrentLeftNode.ParentNode.ID = dr["NodeID"].DbInt();
                    data.CurrentLeftNode.ParentNode.Url = dr["Url"].DbString();
                    data.CurrentLeftNode.ParentNode.Title = dr["Title"].DbString();
                    data.CurrentLeftNode.ParentNode.DisplayOrder = dr["DisplayOrder"].DbInt();
                }

                dr.NextResult();
                while (dr.Read())
                {
                    var node = new Shared.Entities.Sitemap.WebLeftNode();
                    node.ID = dr["NodeID"].DbInt();
                    node.Url = dr["Url"].DbString();
                    node.Title = dr["Title"].DbString();
                    node.DisplayOrder = dr["DisplayOrder"].DbInt();
                    data.CurrentLeftNode.ParentNode.ChildNodes.Add(node);
                }

                dr.NextResult();
                while (dr.Read())
                {
                    var node = new Shared.Entities.Sitemap.WebLeftNode();
                    node.ID = dr["NodeID"].DbInt();
                    node.Url = dr["Url"].DbString();
                    node.Title = dr["Title"].DbString();
                    node.DisplayOrder = dr["DisplayOrder"].DbInt();
                    data.CurrentLeftNode.ChildNodes.Add(node);
                }

                dr.NextResult();
                while (dr.Read())
                {
                    var node = new Shared.Entities.Sitemap.WebLeftNode();
                    node.ID = dr["NodeID"].DbInt();
                    node.Url = dr["Url"].DbString();
                    node.Title = dr["Title"].DbString();
                    node.DisplayOrder = dr["DisplayOrder"].DbInt();
                    data.CurrentLeftNode.ParentNode.ParentNode.ChildNodes.Add(node);
                }

                // Bread Crumb
                dr.NextResult();
                while (dr.Read())
                {
                    var node = new Shared.Entities.Sitemap.Node();
                    node.ID = dr["NodeID"].DbInt();
                    node.Url = dr["Url"].DbString();
                    node.Title = dr["Title"].DbString();
                    data.BreadCrumb.Add(node);
                }
            }

            foreach (var node in topnodes)
            {
                if (node.ID == 1)
                {
                    data.TopHomeNode = node;
                }
                else
                {
                    topnodes.Where(n => n.ID == node.ParentNode.ID).FirstOrDefault().ChildNodes.Add(node);
                }
            }

            foreach (var node in bottomnodes)
            {
                if (node.ID == 1)
                {
                    data.BottomHomeNode = node;
                }
                else
                {
                    bottomnodes.Where(n => n.ID == node.ParentNode.ID).FirstOrDefault().ChildNodes.Add(node);
                }
            }

            return data;
        }
    }
}