﻿using System.Data;
using System.Data.SqlClient;

namespace Adams.Web.Areas.MyAdams.Data.Businesses
{
    public static class Create
    {
        public static MyAdams.Models.Businesses.Create.PageLoad PageLoad(int memberId, int businessId)
        {
            var data = new MyAdams.Models.Businesses.Create.PageLoad()
            { 
                Business = new Shared.Entities.Businesses.Business(),
                Types = new Shared.Entities.Businesses.TypeCollection(),
                Categories = new Shared.Entities.Businesses.CategoryCollection()
            };

            using (SqlDataReader dr = Adams.Web.Data.DB.RunDataReader(@"
IF(@BusinessID > 0)
BEGIN
    DECLARE @TypeID int
    DECLARE @CategoryID int
    SELECT @CategoryID = CategoryID FROM Businesses.Businesses WHERE BusinessID = @BusinessID
    SELECT @TypeID = TypeID FROM Businesses.Categories WHERE CategoryID = @CategoryID

    SELECT P.BusinessID,P.CategoryID,P.MemberID,P.StatusID,P.Name,P.Street,P.Street2,P.City,P.State,P.Zip,P.Phone,P.Description,P.CreatedOn,P.ExpiresOn,M.FirstName,M.LastName,M.Email,T.TypeID
    FROM Businesses.Businesses P INNER JOIN Businesses.Categories C ON P.CategoryID = C.CategoryID INNER JOIN Businesses.Types T ON C.TypeID = T.TypeID INNER JOIN Membership.Members M ON P.MemberID = M.MemberID
    WHERE BusinessID = @BusinessID

    SELECT * FROM Businesses.Categories WHERE TypeID = @TypeID ORDER BY Name
END
SELECT * FROM Businesses.Types ORDER BY Name
", CommandType.Text,
            Adams.Web.Data.DB.CreateParameter("@BusinessID", businessId)))
            {
                if (businessId > 0)
                {
                    dr.Read();
                    data.Business = Adams.Web.Data.DB.InitFromReader<Adams.Shared.Entities.Businesses.Business>(dr, new Adams.Web.Data.ColMap { NameInDb = "BusinessID", NameInObject = "ID" });
                    data.Business.Status = (Shared.Entities.Businesses.Status)dr["StatusID"].DbInt();
                    data.Business.Category = new Shared.Entities.Businesses.Category()
                    {
                        ID = dr["CategoryID"].DbInt(),
                        Type = new Shared.Entities.Businesses.Type()
                        {
                            ID = dr["TypeID"].DbInt()
                        }
                    };
                    data.Business.Member = new Shared.Entities.Membership.Member()
                    {
                        ID = dr["MemberID"].DbInt(),
                        Email = dr["Email"].DbString(),
                        FirstName = dr["FirstName"].DbString(),
                        LastName = dr["LastName"].DbString(),
                    };

                    dr.NextResult();
                    while (dr.Read())
                    {
                        var cat  = Adams.Web.Data.DB.InitFromReader<Adams.Shared.Entities.Businesses.Category>(dr, new Adams.Web.Data.ColMap { NameInDb = "CategoryID", NameInObject = "ID" });
                        data.Categories.Add(cat);
                    }
                    dr.NextResult();
                }
                while (dr.Read())
                {
                    var typ = Adams.Web.Data.DB.InitFromReader<Adams.Shared.Entities.Businesses.Type>(dr, new Adams.Web.Data.ColMap { NameInDb = "TypeID", NameInObject = "ID" });
                    data.Types.Add(typ);
                }
            }

            return data;
        }

        public static Adams.Shared.Entities.Businesses.CategoryCollection GetCategoriesByType(int typeId)
        {
            var data = new Shared.Entities.Businesses.CategoryCollection();

            if (typeId > 0)
            {
                using (SqlDataReader dr = Adams.Web.Data.DB.RunDataReader(@"
SELECT * FROM Businesses.Categories WHERE TypeID = @TypeID ORDER BY Name
", CommandType.Text,
                Adams.Web.Data.DB.CreateParameter("@TypeID", typeId, SqlDbType.Int)))
                {
                    while (dr.Read())
                    {
                        var cat = Adams.Web.Data.DB.InitFromReader<Adams.Shared.Entities.Businesses.Category>(dr, new Adams.Web.Data.ColMap { NameInDb = "CategoryID", NameInObject = "ID" });
                        data.Add(cat);
                    }
                }
            }

            return data;
        }

        public static void Save(MyAdams.Models.Businesses.Create.SaveInput input)
        {
            Adams.Web.Data.DB.Run(@"
IF(@BusinessID > 0)
BEGIN
    UPDATE Businesses.Businesses SET
        CategoryID = @CategoryID
        ,StatusID = @StatusID
        ,Name = @Name
        ,Street = @Street
        ,Street2 = @Street2
        ,City = @City
        ,State = @State
        ,Zip = @Zip
        ,Phone = @Phone
        ,Description = @Description
    WHERE BusinessID = @BusinessID AND MemberID = @MemberID
END
ELSE
BEGIN
    INSERT INTO Businesses.Businesses (
        CategoryID
        ,MemberID
        ,StatusID
        ,Name
        ,Street
        ,Street2
        ,City
        ,State
        ,Zip
        ,Phone
        ,Description
        ,ExpiresOn)
    VALUES (
        @CategoryID
        ,@MemberID
        ,@StatusID
        ,@Name
        ,@Street
        ,@Street2
        ,@City
        ,@State
        ,@Zip
        ,@Phone
        ,@Description
        ,@ExpiresOn)
END
    ", CommandType.Text,
                    Adams.Web.Data.DB.CreateParameter("@BusinessID", input.ID.ToInt()),
                    Adams.Web.Data.DB.CreateParameter("@CategoryID", input.CategoryID.ToInt()),
                    Adams.Web.Data.DB.CreateParameter("@MemberID", input.MemberID.ToInt()),
                    Adams.Web.Data.DB.CreateParameter("@StatusID", input.Status.ToInt()),
                    Adams.Web.Data.DB.CreateParameter("@Name", input.Name.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@Street", input.Street.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@Street2", input.Street2.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@City", input.City.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@State", input.State.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@Zip", input.Zip.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@Phone", input.Phone.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@Description", input.Description.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@ExpiresOn", input.ExpiresOn.ToDateTime()));
        }
    }
}
