﻿using System.Data;
using System.Data.SqlClient;

namespace Adams.Web.Areas.MyAdams.Data.Classifieds
{
    public static class Create
    {
        public static MyAdams.Models.Classifieds.Create.PageLoad PageLoad(int memberId, int PostingId)
        {
            var data = new MyAdams.Models.Classifieds.Create.PageLoad()
            { 
                Posting = new Shared.Entities.Classifieds.Posting(),
                Types = new Shared.Entities.Classifieds.TypeCollection(),
                Categories = new Shared.Entities.Classifieds.CategoryCollection()
            };

            using (SqlDataReader dr = Adams.Web.Data.DB.RunDataReader(@"
IF(@PostingID > 0)
BEGIN
    DECLARE @TypeID int
    DECLARE @CategoryID int
    SELECT @CategoryID = CategoryID FROM Classifieds.Postings WHERE PostingID = @PostingID
    SELECT @TypeID = TypeID FROM Classifieds.Categories WHERE CategoryID = @CategoryID

    SELECT P.PostingID,P.CategoryID,P.MemberID,P.StatusID,P.Title,P.Price,P.Location,P.ReplyTo,P.Description,P.CreatedOn,P.ExpiresOn,M.FirstName,M.LastName,M.Email,T.TypeID
    FROM Classifieds.Postings P INNER JOIN Classifieds.Categories C ON P.CategoryID = C.CategoryID INNER JOIN Classifieds.Types T ON C.TypeID = T.TypeID INNER JOIN Membership.Members M ON P.MemberID = M.MemberID
    WHERE PostingID = @PostingID

    SELECT * FROM Classifieds.Categories WHERE TypeID = @TypeID ORDER BY Name
END
SELECT * FROM Classifieds.Types ORDER BY Name
", CommandType.Text,
            Adams.Web.Data.DB.CreateParameter("@PostingID", PostingId)))
            {
                if (PostingId > 0)
                {
                    dr.Read();
                    data.Posting = Adams.Web.Data.DB.InitFromReader<Adams.Shared.Entities.Classifieds.Posting>(dr, new Adams.Web.Data.ColMap { NameInDb = "PostingID", NameInObject = "ID" });
                    data.Posting.Status = (Shared.Entities.Classifieds.Status)dr["StatusID"].DbInt();
                    data.Posting.Category = new Shared.Entities.Classifieds.Category()
                    {
                        ID = dr["CategoryID"].DbInt(),
                        Type = new Shared.Entities.Classifieds.Type()
                        {
                            ID = dr["TypeID"].DbInt()
                        }
                    };
                    data.Posting.Member = new Shared.Entities.Membership.Member()
                    {
                        ID = dr["MemberID"].DbInt(),
                        Email = dr["Email"].DbString(),
                        FirstName = dr["FirstName"].DbString(),
                        LastName = dr["LastName"].DbString(),
                    };

                    dr.NextResult();
                    while (dr.Read())
                    {
                        var cat  = Adams.Web.Data.DB.InitFromReader<Adams.Shared.Entities.Classifieds.Category>(dr, new Adams.Web.Data.ColMap { NameInDb = "CategoryID", NameInObject = "ID" });
                        data.Categories.Add(cat);
                    }
                    dr.NextResult();
                }
                while (dr.Read())
                {
                    var typ = Adams.Web.Data.DB.InitFromReader<Adams.Shared.Entities.Classifieds.Type>(dr, new Adams.Web.Data.ColMap { NameInDb = "TypeID", NameInObject = "ID" });
                    data.Types.Add(typ);
                }
            }

            return data;
        }

        public static Adams.Shared.Entities.Classifieds.CategoryCollection GetCategoriesByType(int typeId)
        {
            var data = new Shared.Entities.Classifieds.CategoryCollection();

            if (typeId > 0)
            {
                using (SqlDataReader dr = Adams.Web.Data.DB.RunDataReader(@"
SELECT * FROM Classifieds.Categories WHERE TypeID = @TypeID ORDER BY Name
", CommandType.Text,
                Adams.Web.Data.DB.CreateParameter("@TypeID", typeId, SqlDbType.Int)))
                {
                    while (dr.Read())
                    {
                        var cat = Adams.Web.Data.DB.InitFromReader<Adams.Shared.Entities.Classifieds.Category>(dr, new Adams.Web.Data.ColMap { NameInDb = "CategoryID", NameInObject = "ID" });
                        data.Add(cat);
                    }
                }
            }

            return data;
        }

        public static void Save(MyAdams.Models.Classifieds.Create.SaveInput input)
        {
            Adams.Web.Data.DB.Run(@"
IF(@PostingID > 0)
BEGIN
    UPDATE Classifieds.Postings SET
        CategoryID = @CategoryID
        ,StatusID = @StatusID
        ,Title = @Title
        ,Price = @Price
        ,Location = @Location
        ,ReplyTo = @ReplyTo
        ,Description = @Description
    WHERE PostingID = @PostingID AND MemberID = @MemberID
END
ELSE
BEGIN
    INSERT INTO Classifieds.Postings (
        CategoryID
        ,MemberID
        ,StatusID
        ,Title
        ,Price
        ,Location
        ,ReplyTo
        ,Description
        ,ExpiresOn)
    VALUES (
        @CategoryID
        ,@MemberID
        ,@StatusID
        ,@Title
        ,@Price
        ,@Location
        ,@ReplyTo
        ,@Description
        ,@ExpiresOn)
END
    ", CommandType.Text,
                    Adams.Web.Data.DB.CreateParameter("@PostingID", input.ID.ToInt()),
                    Adams.Web.Data.DB.CreateParameter("@CategoryID", input.CategoryID.ToInt()),
                    Adams.Web.Data.DB.CreateParameter("@MemberID", input.MemberID.ToInt()),
                    Adams.Web.Data.DB.CreateParameter("@StatusID", input.Status.ToInt()),
                    Adams.Web.Data.DB.CreateParameter("@Title", input.Title.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@Price", input.Price.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@Location", input.Location.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@ReplyTo", input.ReplyTo.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@Description", input.Description.ToNString()),
                    Adams.Web.Data.DB.CreateParameter("@ExpiresOn", input.ExpiresOn.ToDateTime()));
        }
    }
}
